package com.jtxt.demo.controller;

import com.jtxt.demo.jwt.JwtTokenProvider;
import com.jtxt.demo.model.LearningMaterial;
import com.jtxt.demo.model.User;
import com.jtxt.demo.service.GroupService;
import com.jtxt.demo.service.LearningMaterialService;
import com.jtxt.demo.service.PointsHistoryService;
import com.jtxt.demo.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.web.bind.annotation.*;

import java.security.Principal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;

import static com.jtxt.demo.util.Constants.CHINA_ZONE;

@RestController
public class UserController {

    @Autowired
    private JwtTokenProvider tokenProvider;

    @Autowired
    private UserService userService;
    @Autowired
    private LearningMaterialService materialService;
    @Autowired
    private PointsHistoryService pointsHistoryService;
    @Autowired
    private GroupService groupService;

    // 删除某个用户
    @DeleteMapping("/admin/users")
    public ResponseEntity<?> deleteUser(@RequestBody User user){
        userService.deleteUser(user.getUserName());
        return new ResponseEntity<>(HttpStatus.OK);
    }
    // 获取所有用户
    @GetMapping("/admin/users")
    public ResponseEntity<?> findAllUsers(){
        // todo: pagination
        return new ResponseEntity<>(userService.findAllUsers(), HttpStatus.OK);
    }

    // 用户注册
    @PostMapping("/user/register")
    public ResponseEntity<?> register(@RequestBody User user){
        if(userService.findByUserName(user.getUserName())!=null){
            return new ResponseEntity<>(HttpStatus.CONFLICT);
        }
        //default role.
        user.setRole(User.Role.USER);
        user.setStatus(User.UserStatus.ACTIVE); // set to Active automatically
        // testing token
        user.setToken("test");
        return new ResponseEntity<>(userService.createUser(user), HttpStatus.CREATED);
    }

    // 用户详情界面
    @GetMapping("/user/{userName}")
    public ResponseEntity<?> getUser(@PathVariable String userName){
        User user = userService.findByUserName(userName);
        if(user == null){
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(user, HttpStatus.OK);
    }

    // 修改用户属性
    @PutMapping("/user/{userName}")
    public ResponseEntity<?> updateUser(@PathVariable String userName, @RequestBody User updatedUser){
        User existUser = userService.findByUserName(userName);
        if (existUser == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedUser.setUserName(userName);
        return new ResponseEntity<>(userService.updateUser(updatedUser), HttpStatus.OK);
    }

    // 用户登录
    @GetMapping("/user/login")
    public ResponseEntity<?> getUser(Principal principal){
        //principal = httpServletRequest.getUserPrincipal.
        if(principal == null){
            //logout will also use here so we should return ok http status.
            return ResponseEntity.ok(principal);
        }
        UsernamePasswordAuthenticationToken authenticationToken =
                (UsernamePasswordAuthenticationToken) principal;
        User user = userService.findByUserName(authenticationToken.getName());
        user.setToken(tokenProvider.generateToken(authenticationToken));
        userService.loginUser(user);
        return new ResponseEntity<>(user, HttpStatus.OK);
    }
    @GetMapping("/user/{userName}/favorites")
    public ResponseEntity<?> showFavorites(@PathVariable String userName){
        User user = userService.findByUserName(userName);
        if(user == null){
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(userService.showFavorites(user), HttpStatus.OK);
    }
    @GetMapping("/user/{userName}/points")
    public ResponseEntity<?> showTotalPoints(@PathVariable String userName) {
        return new ResponseEntity<>(userService.getTotalPoints(userName), HttpStatus.OK);
    }

    @PostMapping("/user/{userName}/favorites")
    public ResponseEntity<?> addFavorite(@PathVariable String userName, @RequestBody LearningMaterial material){
        User user = userService.findByUserName(userName);
        if(user == null){
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        LearningMaterial materialInDb = materialService.getMaterialById(material.getId());
        if (materialInDb == null) {
            return new ResponseEntity<>("Material Not Found", HttpStatus.NOT_FOUND);
        }
        boolean rst = userService.addFavorite(material, user);
        return new ResponseEntity<>(rst? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }
    @DeleteMapping("/user/{userName}/favorites")
    public ResponseEntity<?> removeFavorite(@PathVariable String userName, @RequestBody LearningMaterial material){
        User user = userService.findByUserName(userName);
        if(user == null){
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        LearningMaterial materialInDb = materialService.getMaterialById(material.getId());
        if (materialInDb == null) {
            return new ResponseEntity<>("Material Not Found", HttpStatus.NOT_FOUND);
        }
        boolean rst = userService.removeFavorite(material, user);
        return new ResponseEntity<>(rst? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }
    @GetMapping("/user/{userName}/daily-points")
    public ResponseEntity<?> getDailyPoints(@PathVariable String userName) {
        User user = userService.findByUserName(userName);
        if(user == null){
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        LocalTime midnight = LocalTime.MIDNIGHT;
        LocalDate today = LocalDate.now(CHINA_ZONE);
        LocalDateTime todayMidnight = LocalDateTime.of(today, midnight);
        int points = pointsHistoryService.showPoints(userName, todayMidnight, LocalDateTime.now(CHINA_ZONE));
        return new ResponseEntity<>(points, HttpStatus.OK);
    }
}