package com.jtxt.demo.service;

import com.jtxt.demo.model.*;
import com.jtxt.demo.repository.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.aggregation.SampleOperation;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.jtxt.demo.util.Constants.CHINA_ZONE;

@Service
public class ExamServiceImpl implements ExamService {
    @Autowired
    private ExamRepository examRepository;
    @Autowired
    private ExamHistoryRepository examHistoryRepository;
    @Autowired
    private ExamToQuestionRepository examToQuestionRepository;
    @Autowired
    private QuestionRepository questionRepository;
    @Autowired
    private ExamTargetGroupRepository examTargetGroupRepository;
    @Autowired
    private ExamTargetUserRepository examTargetUserRepository;
    @Autowired
    private GroupToUserRepository groupToUserRepository;
    @Autowired
    private GroupRepository groupRepository;
    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public Exam getExamByName(String name) {
        return examRepository.findByName(name);
    }

    @Override
    public Exam getExamById(String id) {
        return examRepository.findById(id).orElse(null);
    }

    @Override
    public List<Exam> showAllAvailableExams(User user) {
        List<Exam> rst = new ArrayList<>();
        List<Exam> availableExamsForGroup = groupToUserRepository.findAllByUserId(user.getUserName())
                .stream()
                .map(g -> examTargetGroupRepository.findAllByGroupId(g.getGroupId()))
                .flatMap(Collection::stream)
                .map(etg -> examRepository.findById(etg.getExamId()))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toList());
        List<Exam> availableExamsForUser = examTargetUserRepository.findAllByUserId(user.getUserName())
                .stream()
                .map(etu -> examRepository.findById(etu.getExamId()))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toList());
        rst.addAll(availableExamsForGroup);
        rst.addAll(availableExamsForUser);
        return rst;
    }

    @Override
    public List<Exam> showAllExams() {
        return examRepository.findAll();
    }

    @Override
    public Exam createExam(Exam exam) {
        return examRepository.insert(exam);
    }

    @Override
    public Exam editExam(Exam updatedExam) {
        Exam examInDb = examRepository.findById(updatedExam.getId()).orElse(null);
        if (updatedExam.getDeadline() != null) {
            examInDb.setDeadline(updatedExam.getDeadline());
        }
        if (updatedExam.getDescription() != null) {
            examInDb.setDescription(updatedExam.getDescription());
        }
        if (updatedExam.getDuration() != 0) {
            examInDb.setDuration(updatedExam.getDuration());
        }
        if (updatedExam.getName() != null) {
            examInDb.setName(updatedExam.getName());
        }
        if (updatedExam.getStartTime() != null) {
            examInDb.setStartTime(updatedExam.getStartTime());
        }
        if (updatedExam.getUserId() != null) {
            examInDb.setUserId(updatedExam.getUserId());
        }
        return examRepository.save(examInDb);
    }

    @Override
    public boolean deleteExam(Exam exam) {
        examRepository.delete(exam);
        return true;
    }

    @Override
    public List<String> randomlyPickQuestions(int number) {
        SampleOperation matchStage = Aggregation.sample(5);
        Aggregation aggregation = Aggregation.newAggregation(matchStage);
        AggregationResults<Question> questions = mongoTemplate.aggregate(aggregation, "question", Question.class);
        Iterator<Question> questionIterator = questions.iterator();
        List<String> list = new ArrayList<>();
        while (questionIterator.hasNext()) {
            list.add(questionIterator.next().getId());
        }
        return list;
    }

    @Override
    public List<String> startExam(Exam exam, User user) {
        ExamHistory examHistory = new ExamHistory();
        examHistory.setExamId(exam.getId());
        examHistory.setStartTime(LocalDateTime.now(CHINA_ZONE));
        examHistory.setUserId(user.getUserName());
        examHistoryRepository.insert(examHistory);
        return examToQuestionRepository
                .findAllByExamId(exam.getId()).stream()
                .map(x -> x.getQuestionId())
                .collect(Collectors.toList());
    }

    @Override
    public ExamHistory endExam(ExamHistory examHistory) {
        ExamHistory examHistoryInDb = examHistoryRepository.findByUserIdAndExamId(examHistory.getUserId(), examHistory.getExamId());

        examHistoryInDb.setEndTime(LocalDateTime.now(CHINA_ZONE));
        examHistoryInDb.setUserAnswers(examHistory.getUserAnswers());
        if (examHistory.getPoints() == 0) {
            // compare the answers and do calculations
        } else {
            examHistoryInDb.setPoints(examHistory.getPoints());
        }
        examHistoryRepository.save(examHistoryInDb);
        return examHistoryInDb;
    }

    @Override
    @Transactional
    public boolean addQuestionsToExam(Exam exam, List<ExamToQuestion> examToQuestions) {
        examToQuestions.stream().forEach(q -> q.setExamId(exam.getId()));
        examToQuestionRepository.insert(examToQuestions);
        return true;
    }

    @Override
    public boolean addQuestionToExam(Exam exam, Question question) {
        examToQuestionRepository.insert(new ExamToQuestion(exam.getId(), question.getId()));
        return true;
    }

    @Override
    public boolean removeQuestionFromExam(Exam exam, Question question) {
        ExamToQuestion relation = examToQuestionRepository.findByExamIdAndQuestionId(exam.getId(), question.getId());
        examToQuestionRepository.delete(relation);
        return true;
    }

    @Override
    public List<ExamHistory> showExamHistoriesForUser(String userName) {
        return examHistoryRepository.findAllByUserId(userName);
    }
}
