package com.jtxt.demo.service;

import com.jtxt.demo.model.*;
import com.jtxt.demo.repository.*;
import com.jtxt.demo.util.CacheStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import static com.jtxt.demo.util.Constants.CHINA_ZONE;

@Service
public class LearningMaterialServiceImpl implements LearningMaterialService {
    @Autowired
    private LearningMaterialRepository repository;
    @Autowired
    private LikeHistoryRepository likeHistoryRepository;
    @Autowired
    private ReadHistoryRepository readHistoryRepository;
    @Autowired
    private CommentRepository commentRepository;
    @Autowired
    private CacheStore<LearningMaterial> materialCache;

    @Override
    public LearningMaterial getMaterialByName(String name) {
        return repository.findByName(name);
    }

    @Override
    public LearningMaterial getMaterialById(String id) {
        LearningMaterial material = materialCache.get(id);
        if (material == null) {
            material = repository.findById(id).orElse(null);
            if (material != null) materialCache.add(id, material);
        }
        return material;
    }

    @Override
    public boolean deleteMaterial(LearningMaterial material) {
        repository.delete(material);
        return true;
    }

    @Override
    public LearningMaterial editMaterial(LearningMaterial updatedMaterial) {
        LearningMaterial materialInDb = getMaterialById(updatedMaterial.getId());
        if (updatedMaterial.getName() != null) {
            materialInDb.setName(updatedMaterial.getName());
        }
        if (updatedMaterial.getDescription() != null) {
            materialInDb.setDescription(updatedMaterial.getDescription());
        }
        if (updatedMaterial.getContents() != null) {
            materialInDb.setContents(updatedMaterial.getContents());
            materialInDb.setReadTimeInSec(getReadTimeInSec(updatedMaterial));
        }
        if (updatedMaterial.getType() != null) {
            materialInDb.setType(updatedMaterial.getType());
        }
        if (updatedMaterial.getStatus() != null) {
            materialInDb.setStatus(updatedMaterial.getStatus());
        }
        return repository.save(materialInDb);
    }

    @Override
    @Transactional
    public void likeMaterial(LearningMaterial material, User user, boolean isLike) {

        material.setLikesCount(isLike ? material.getLikesCount() + 1 : material.getLikesCount() - 1);
        repository.save(material);
        LikeHistory likeHistory;
        if (isLike) {
            likeHistory = new LikeHistory();
            likeHistory.setMaterialId(material.getId());
            likeHistory.setUserId(user.getUserName());
            likeHistoryRepository.insert(likeHistory);
        } else {
            likeHistory = likeHistoryRepository.findByMaterialIdAndUserId(material.getId(), user.getUserName()).orElse(null);
            if (likeHistory != null) {
                likeHistoryRepository.delete(likeHistory);
            }
        }
    }

    @Override
    @Transactional
    public void readMaterial(LearningMaterial material, User user) {
        material.setReaderCount(material.getReaderCount() + 1);
        repository.save(material);
        ReadHistory readHistory = new ReadHistory();
        readHistory.setMaterialId(material.getId());
        readHistory.setUserId(user.getUserName());
        readHistoryRepository.insert(readHistory);
    }
    @Override
    public boolean isMaterialReadable(LearningMaterial material, String userName) {
        Optional<ReadHistory> readHistory = readHistoryRepository
                .findByMaterialIdAndUserId(material.getId(), userName);
        return readHistory.isEmpty();
    }

    @Override
    public boolean isMaterialLikeable(LearningMaterial material, String userName) {
        Optional<LikeHistory> likeHistory = likeHistoryRepository
                .findByMaterialIdAndUserId(material.getId(), userName);
        return likeHistory.isEmpty();
    }

    @Override
    public void commentMaterial(Comment comment) {
        comment.setCreatedTime(LocalDateTime.now(CHINA_ZONE));
        commentRepository.insert(comment);
    }

    @Override
    public List<Comment> showComments(LearningMaterial material) {
        return commentRepository.findAllByMaterialId(material.getId());
    }

    @Override
    public List<LearningMaterial> searchMaterialsByName(String name) {
        return repository.findByNameLike(name);
    }

    @Override
    public LearningMaterial createMaterial(LearningMaterial material) {
        material.setStatus(ProvisionStatus.SUBMITTED);
        material.setCreatedTime(LocalDateTime.now(CHINA_ZONE));
        material.setReadTimeInSec(getReadTimeInSec(material));
        return repository.insert(material);
    }

    // Get Read Time based on material content length
    // assume normal speed is 10 words per second.
    private int getReadTimeInSec(LearningMaterial material) {
        String contents = material.getContents();
        return contents.length() / 10;
    }
}
